package com.adventurer_engine.client.model.tmt;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import atc;
import bcu;
import com.adventurer_engine.client.model.tmt.Angle3D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import ls;

public class Bone {
  protected Angle3D neutralAngles;
  
  public Angle3D relativeAngles;
  
  protected Angle3D absoluteAngles;
  
  private atc positionVector;
  
  private float length;
  
  private com.adventurer_engine.client.model.tmt.Bone parentNode;
  
  protected ArrayList<com.adventurer_engine.client.model.tmt.Bone> childNodes;
  
  private ArrayList<bcu> models;
  
  private Map<bcu, Angle3D> modelBaseRot;
  
  private float offsetX;
  
  private float offsetY;
  
  private float offsetZ;
  
  public Bone(float x, float y, float z, float l) {
    this.neutralAngles = new Angle3D(x, y, z);
    this.relativeAngles = new Angle3D(0.0F, 0.0F, 0.0F);
    this.absoluteAngles = new Angle3D(0.0F, 0.0F, 0.0F);
    this.positionVector = atc.a(0.0D, 0.0D, 0.0D);
    this.length = l;
    this.childNodes = new ArrayList<com.adventurer_engine.client.model.tmt.Bone>();
    this.models = new ArrayList<bcu>();
    this.modelBaseRot = new HashMap<bcu, Angle3D>();
    this.parentNode = null;
    this.offsetX = 0.0F;
    this.offsetY = 0.0F;
    this.offsetZ = 0.0F;
    this.positionVector = atc.a(0.0D, 0.0D, 0.0D);
  }
  
  public Bone(float xOrig, float yOrig, float zOrig, float xRot, float yRot, float zRot, float l) {
    this(xRot, yRot, zRot, l);
    this.positionVector = setOffset(xOrig, yOrig, zOrig);
  }
  
  public Bone(float x, float y, float z, float l, com.adventurer_engine.client.model.tmt.Bone parent) {
    this(x, y, z, l);
    attachBone(parent);
  }
  
  public void detachBone() {
    this.parentNode.childNodes.remove(this);
    this.parentNode = null;
  }
  
  public void attachBone(com.adventurer_engine.client.model.tmt.Bone parent) {
    if (this.parentNode != null)
      detachBone(); 
    this.parentNode = parent;
    parent.addChildBone(this);
    this.offsetX = parent.offsetX;
    this.offsetY = parent.offsetY;
    this.offsetZ = parent.offsetZ;
    resetOffset();
  }
  
  public atc setOffset(float x, float y, float z) {
    if (this.parentNode != null) {
      atc vector = this.parentNode.setOffset(x, y, z);
      this.offsetX = (float)vector.c;
      this.offsetY = (float)vector.d;
      this.offsetZ = (float)vector.e;
      return vector;
    } 
    this.offsetX = x;
    this.offsetY = y;
    this.offsetZ = z;
    resetOffset(true);
    return atc.a(x, y, z);
  }
  
  public void resetOffset() {
    resetOffset(false);
  }
  
  public void resetOffset(boolean doRecursive) {
    if (this.parentNode != null) {
      this.positionVector = atc.a(0.0D, 0.0D, this.parentNode.length);
      this.parentNode.setVectorRotations(this.positionVector);
      this.positionVector.c += this.parentNode.positionVector.c;
      this.positionVector.d += this.parentNode.positionVector.d;
      this.positionVector.e += this.parentNode.positionVector.e;
    } 
    if (doRecursive && !this.childNodes.isEmpty())
      for (int index = 0; index < this.childNodes.size(); index++)
        ((com.adventurer_engine.client.model.tmt.Bone)this.childNodes.get(index)).resetOffset(doRecursive);  
  }
  
  public void setNeutralRotation(float x, float y, float z) {
    this.neutralAngles.angleX = x;
    this.neutralAngles.angleY = y;
    this.neutralAngles.angleZ = z;
  }
  
  public com.adventurer_engine.client.model.tmt.Bone getRootParent() {
    if (this.parentNode == null)
      return this; 
    return this.parentNode.getRootParent();
  }
  
  public void addModel(bcu model) {
    addModel(model, false);
  }
  
  public void addModel(bcu model, boolean inherit) {
    addModel(model, 0.0F, 0.0F, 0.0F, inherit);
  }
  
  public void addModel(bcu model, boolean inherit, boolean isUpright) {
    addModel(model, 0.0F, 0.0F, 0.0F, inherit, isUpright);
  }
  
  public void addModel(bcu model, float x, float y, float z) {
    addModel(model, x, y, z, false);
  }
  
  public void addModel(bcu model, float x, float y, float z, boolean inherit) {
    addModel(model, x, y, z, inherit, false);
  }
  
  public void addModel(bcu model, float x, float y, float z, boolean inherit, boolean isUpright) {
    if (inherit) {
      x += this.neutralAngles.angleX + (isUpright ? 1.5707964F : 0.0F);
      y += this.neutralAngles.angleY;
      z += this.neutralAngles.angleZ;
    } 
    this.models.add(model);
    this.modelBaseRot.put(model, new Angle3D(x, y, z));
  }
  
  public void removeModel(bcu model) {
    this.models.remove(model);
    this.modelBaseRot.remove(model);
  }
  
  public Angle3D getAbsoluteAngle() {
    return new Angle3D(this.absoluteAngles.angleX, this.absoluteAngles.angleY, this.absoluteAngles.angleZ);
  }
  
  public atc getPosition() {
    return atc.a(this.positionVector.c, this.positionVector.d, this.positionVector.e);
  }
  
  protected void addChildBone(com.adventurer_engine.client.model.tmt.Bone bone) {
    this.childNodes.add(bone);
  }
  
  public void prepareDraw() {
    if (this.parentNode != null) {
      this.parentNode.prepareDraw();
    } else {
      setAbsoluteRotations();
      setVectors();
    } 
  }
  
  public void setRotations(float x, float y, float z) {
    this.relativeAngles.angleX = x;
    this.relativeAngles.angleY = y;
    this.relativeAngles.angleZ = z;
  }
  
  protected void setAbsoluteRotations() {
    this.absoluteAngles.angleX = this.relativeAngles.angleX;
    this.absoluteAngles.angleY = this.relativeAngles.angleY;
    this.absoluteAngles.angleZ = this.relativeAngles.angleZ;
    for (int i = 0; i < this.childNodes.size(); i++)
      ((com.adventurer_engine.client.model.tmt.Bone)this.childNodes.get(i)).setAbsoluteRotations(this.absoluteAngles.angleX, this.absoluteAngles.angleY, this.absoluteAngles.angleZ); 
  }
  
  protected void setAbsoluteRotations(float x, float y, float z) {
    this.relativeAngles.angleX += x;
    this.relativeAngles.angleY += y;
    this.relativeAngles.angleZ += z;
    for (int i = 0; i < this.childNodes.size(); i++)
      ((com.adventurer_engine.client.model.tmt.Bone)this.childNodes.get(i)).setAbsoluteRotations(this.absoluteAngles.angleX, this.absoluteAngles.angleY, this.absoluteAngles.angleZ); 
  }
  
  protected void setVectorRotations(atc vector) {
    float x = this.neutralAngles.angleX + this.absoluteAngles.angleX;
    float y = this.neutralAngles.angleY + this.absoluteAngles.angleY;
    float z = this.neutralAngles.angleZ + this.absoluteAngles.angleZ;
    setVectorRotations(vector, x, y, z);
  }
  
  protected void setVectorRotations(atc vector, float xRot, float yRot, float zRot) {
    float x = xRot;
    float y = yRot;
    float z = zRot;
    float xC = ls.b(x);
    float xS = ls.a(x);
    float yC = ls.b(y);
    float yS = ls.a(y);
    float zC = ls.b(z);
    float zS = ls.a(z);
    double xVec = vector.c;
    double yVec = vector.d;
    double zVec = vector.e;
    double xy = xC * yVec - xS * zVec;
    double xz = xC * zVec + xS * yVec;
    double yz = yC * xz - yS * xVec;
    double yx = yC * xVec + yS * xz;
    double zx = zC * yx - zS * xy;
    double zy = zC * xy + zS * yx;
    xVec = zx;
    yVec = zy;
    zVec = yz;
    vector.c = xVec;
    vector.d = yVec;
    vector.e = zVec;
  }
  
  protected void addVector(atc destVec, atc srcVec) {
    destVec.c += srcVec.c;
    destVec.d += srcVec.d;
    destVec.e += srcVec.e;
  }
  
  protected void setVectors() {
    atc tempVec = atc.a(0.0D, 0.0D, this.length);
    this.positionVector = atc.a(this.offsetX, this.offsetY, this.offsetZ);
    addVector(tempVec, this.positionVector);
    setVectorRotations(tempVec);
    for (int i = 0; i < this.childNodes.size(); i++)
      ((com.adventurer_engine.client.model.tmt.Bone)this.childNodes.get(i)).setVectors(tempVec); 
  }
  
  protected void setVectors(atc vector) {
    this.positionVector = vector;
    atc tempVec = atc.a(0.0D, 0.0D, this.length);
    setVectorRotations(tempVec);
    addVector(tempVec, vector);
    for (int i = 0; i < this.childNodes.size(); i++)
      ((com.adventurer_engine.client.model.tmt.Bone)this.childNodes.get(i)).setVectors(tempVec); 
  }
  
  public void setAnglesToModels() {
    int i;
    for (i = 0; i < this.models.size(); i++) {
      bcu currentModel = this.models.get(i);
      Angle3D baseAngles = this.modelBaseRot.get(currentModel);
      currentModel.f = baseAngles.angleX + this.absoluteAngles.angleX;
      currentModel.g = baseAngles.angleY + this.absoluteAngles.angleY;
      currentModel.h = baseAngles.angleZ + this.absoluteAngles.angleZ;
      currentModel.c = (float)this.positionVector.c;
      currentModel.d = (float)this.positionVector.d;
      currentModel.e = (float)this.positionVector.e;
    } 
    for (i = 0; i < this.childNodes.size(); i++)
      ((com.adventurer_engine.client.model.tmt.Bone)this.childNodes.get(i)).setAnglesToModels(); 
  }
}
